<?php

namespace Drupal\text_file_viewer\Plugin\Field\FieldFormatter;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Plugin\Field\FieldFormatter\FileFormatterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'text_file_viewer' formatter.
 *
 * @FieldFormatter(
 *   id = "text_file_viewer",
 *   label = @Translation("Text file viewer"),
 *   field_types = {
 *     "file"
 *   }
 * )
 */
class TextFileViewer extends FileFormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The File System service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * Constructs a PDBFieldViewer instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the viewer.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the viewer is associated.
   * @param array $settings
   *   The viewer settings.
   * @param string $label
   *   The viewer label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration interface.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger service.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   */
  public function __construct($plugin_id,
                              $plugin_definition,
                              FieldDefinitionInterface $field_definition,
                              array $settings,
                              $label,
                              $view_mode,
                              array $third_party_settings,
                              ConfigFactoryInterface $configFactory,
                              Messenger $messenger,
                              FileSystemInterface $fileSystem,
                              FileUrlGeneratorInterface $file_url_generator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);

    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->fileSystem = $fileSystem;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container,
                                array $configuration,
                                $plugin_id,
                                $plugin_definition) {
    return new static(
          $plugin_id,
          $plugin_definition,
          $configuration['field_definition'],
          $configuration['settings'],
          $configuration['label'],
          $configuration['view_mode'],
          $configuration['third_party_settings'],
          $container->get('config.factory'),
          $container->get('messenger'),
          $container->get('file_system'),
          $container->get('file_url_generator')
      );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'show_file_link' => 0,
      'show_file_size' => 0,
      'max_file_size' => 0,
      'return_empty' => 0,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);

    $form['show_file_link'] = [
      '#title' => $this->t("Show the file's name with a link to the file."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('show_file_link'),
    ];

    $form['show_file_size'] = [
      '#title' => $this->t("Show the file's size."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('show_file_size'),
    ];

    $form['max_file_size'] = [
      '#title' => $this->t('Max file size'),
      '#description' => $this->t('Limit this file formatter to a maximum file size in bytes. 0 indicates unlimited.'),
      '#type' => 'textfield',
      '#attributes' => [
        ' type' => 'number',
      ],
      '#element_validate' => [[$this, 'validateSize']],
      '#required' => TRUE,
      '#default_value' => $this->getSetting('max_file_size'),
    ];

    $form['return_empty'] = [
      '#title' => $this->t('Disable all content if file format not recognized.'),
      '#description' => $this->t("If a file's content is not recognized and cannot be drawn, show nothing."),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('return_empty'),
    ];

    return $form;
  }

  /**
   * Validates width form element.
   *
   * @param array $element
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $complete_form
   *   The complete form structure.
   */
  public static function validateSize(array &$element, FormStateInterface $form_state, array &$complete_form): void {
    $size = $element['#value'];

    if (!is_numeric($size) || $size < 0) {
      $form_state->setErrorByName("max_file_size", "The max file size for TextFileViewer must be greater than or equal to 0.");
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();
    $summary[] = "Display syntax highlighted text.";
    if ($settings['return_empty'] == 1) {
      $summary[] = $this->t('Empty if file type is unrecognized.');
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $settings = $this->getSettings();

    // Create an empty render array.
    $elements = [];

    $config = $this->configFactory->get('text_file_viewer.settings');

    $support_all = $config->get("support_all");

    $supported_extensions = explode(" ", $config->get('supported_extensions'));

    $supported_mimes = explode(" ", $config->get('supported_mimes'));

    // Loop through the entities we need to process and create a render
    // array for each one.
    /** @var \Drupal\Core\Field\EntityReferenceFieldItemListInterface $items */
    $fileEntities = $this->getEntitiesToView($items, $langcode);
    foreach ($fileEntities as $delta => $file) {
      $markup = '';
      $load_libraries = FALSE;

      // Get the server file system's real path to the file.
      /** @var \Drupal\file\FileInterface $file */
      $path = $this->fileSystem->realpath($file->getFileUri());
      if ($path === FALSE) {
        $markup = '<p>' . $this->t('A server error occurred. The file could not be found.') . '</p>';
        if ($settings['return_empty'] == 1) {
          return [];
        }
      }

      // Get the user-visible file name.
      $mimetype = $file->getMimeType();
      $filename = $file->getFilename();
      // $filepath = file_create_url($file->getFileUri());
      // /** @var \Drupal\Core\GeneratedUrl $filepath */
      $filepath = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());

      // Get the file name extension and see if it is supported by
      // any of the known file formats.
      $ext = pathinfo($filename, PATHINFO_EXTENSION);

      if ($markup === '' && $support_all || ($ext && (in_array($ext, $supported_extensions) || in_array($mimetype, $supported_mimes)))) {
        if ($settings['max_file_size'] > 0) {
          if ($file->getSize() > $settings['max_file_size']) {
            $markup .= '<a href="' . $filepath . '">' . $filename . '</a>';
            $this->messenger->addError('Unable to render file with TextFileViewer as file exceeds ' . $settings['max_file_size'] . ' bytes.');
          }
        }

        if ($markup === '') {
          if ($settings['show_file_link'] == 1) {
            if ($settings['show_file_size'] == 1) {
              $markup .= '<p><a href="' . $filepath . '">' . $filename . '</a> (' . $file->getSize() . ' bytes)</p>';
            }
            else {
              $markup .= '<a href="' . $filepath . '">' . $filename . '</a>';
            }
          }
          else {
            if ($settings['show_file_size'] == 1) {
              $markup .= '<p>File Size: ' . $file->getSize() . ' bytes</p>';
            }
          }

          $code = '';
          $array = file($file->getFileUri());

          foreach ($array as $item) {
            // Running js code from string
            // Running php code from string
            // xss
            // sql injection.
            $code .= htmlentities($item, ENT_QUOTES | ENT_HTML5, 'UTF-8');
          }

          if (in_array($ext, $supported_extensions)) {
            $markup .= '<pre><code class="line-numbers language-' . $this->convertExtensionToPrismClassName($ext) . '">' . $code . '</code></pre>';
          }
          else {
            $markup .= '<pre><code class="line-numbers language-' . $this->convertExtensionToPrismClassName("txt") . '">' . $code . '</code></pre>';

          }

          $load_libraries = TRUE;
        }
      }

      if ($markup === '' && (!$ext || !in_array($ext, $supported_extensions))) {
        $markup = '<a href="' . $filepath . '">' . $filename . '</a>';
        if ($settings['return_empty'] == 1) {
          return [];
        }
      }

      if ($load_libraries) {
        $library = 'text_file_viewer/text_file_viewer.' . $config->get('use_cdn');

        $elements[$delta] = [
          '#markup' => $markup,
          '#attached' => [
            'library' => [
              $library,
            ],
          ],
        ];
      }
      else {
        $elements[$delta] = [
          '#markup' => $markup,
        ];
      }

    }

    return $elements;
  }

  /**
   * Map the extension to prime class name.
   */
  private function convertExtensionToPrismClassName($ext): string {
    $hTable = [
      "htm" => "markup",
      "html" => "markup",
      "xml" => "markup",
      "svg" => "markup",
      "md" => "markup",
      "mathml" => "markup",
      "css" => "css",
      "clike" => "clike",
      "pl" => "perl",
      "cpp" => "cpp",
      "hpp" => "cpp",
      "m" => "matlab",
      "mat" => "matlab",
      "sh" => "bash",
      "csh" => "bash",
      "ps1" => "powershell",
      "bat" => "powershell",
      "txt" => "textile",
      "csv" => "textile",
      "tsv" => "textile",
      "cs" => "csharp",
      "conf" => "apacheconf",
    ];

    return array_key_exists($ext, $hTable) ? $hTable[$ext] : $ext;
  }

}
